/*
   Plogicmp - ICMP logging routines
   --------------------------------------------------------------------
   Protolog - The Internet Protocols logger

   Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

void usage( char *progname);

int main( int argc, char *argv[])
{
 FILE *fplog;          /* Simple log file pointer            */
 FILE *fpraw;          /* Raw log file poinetr               */
 char buf[     65535]; /* The buffer we use to read a packet */
 char type[      256]; /* ICMP type string                   */
 struct iphdr iph;     /* IP header                          */
 struct icmphdr ich;   /* ICMP header                        */
 struct in_addr addrs;
 struct sockaddr_in name;
 struct tm *ltm;       /* Used by localtime(3)               */
 time_t t;             /* Used by time(2)                    */
 char *ti;             /* Used by ctime(3)                   */
 int flag_log;         /* Make a simple human readeable log  */
 int flag_raw;         /* Log time + raw data                */
 int ch;               /* For command line parsing           */
 int sock;             /* socket fd                          */
 int length;           /* size of struct sockaddr_in name    */
 int iphlen;           /* IP header length (Used to skip     */
                       /* options)                           */
 int bread;            /* Bytes read                         */

 /* Check if we are root */
 check_uid( argv[ 0]);

 flag_log          = 0;
 flag_raw          = 0;
 flag_dont_resolve = 1;

 if( argc < 2) 
     usage( argv[ 0]);

 /* Parse command line */

 while(( ch = getopt( argc, argv, "lwr")) != EOF){
         switch( ch) {
                case 'l': flag_log = 1; 
                          break;
                case 'w': flag_raw = 1; 
                          break;
                case 'r': flag_dont_resolve = 0;
                          break;
                default : usage( argv[ 0]);
         }
 }

 if( !flag_log && !flag_raw)
     usage( argv[ 0]);

 /* Open a raw icmp socket in order to receive all ICMP packets */

 if( (sock = socket( AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0) {
     fprintf( stderr, "%s: %s\n", argv[ 0], strerror( errno));
     exit( -1);
 }

 name.sin_family      = AF_INET;
 name.sin_addr.s_addr = INADDR_ANY;
 name.sin_port        = 0;

 if( bind( sock, ( struct sockaddr *) &name, sizeof name ) < 0) {
     fprintf( stderr, "%s: %s\n", argv[ 0], strerror( errno));
     exit( -1);
 }

 length = sizeof( name);

 if( getsockname( sock, (struct sockaddr *) &name, &length) < 0) {
     fprintf( stderr, "%s: %s\n", argv[ 0], strerror( errno));
     exit( -1);
 }

 printf( "\nProtolog Release %s - ", PRO_VERSION);
 printf( "plogicmp (ICMP logger)\n\n");

 gobackground();
 init_signals();

 if( flag_log) {
     fplog = fopen( PRO_ICMP_LOG, "a+");

     t  = time( NULL);
     ti = ctime( &t);

     /* Write log headings */
     fprintf( fplog, "%s\n", replicate( '-', 80));
     fprintf( fplog, "Protolog ICMP logger - Begins at %-24.24s\n", ti);
     fprintf( fplog, "%s\n", replicate( '-', 80));
     fprintf( fplog, "-      Date      -   Source IP   -    Domain Name    -     ICMP type      -code-\n");
     fprintf( fplog, "%s\n", replicate( '-', 80));

     fclose( fplog);
 }

 while( 1) {
        bread = read( sock, buf, sizeof( buf));

        if( bread <= 0)
            continue;

        /* Fill the IP header */
        memcpy( &iph, buf, sizeof( iph));

        /* Skip IP options */
        iphlen  = iph.ihl << 2;
        iphlen -=20;

        /* Fill the ICMP header */
        memcpy( &ich, buf + sizeof( iph) + iphlen, sizeof( ich));

        t = time( NULL);

        /* Make a simple human readeable log */
        if( flag_log) {
            fplog = fopen( PRO_ICMP_LOG, "a+");

            ltm = localtime( &t);

            fprintf( fplog, "%02d:%02d:%02d %02d/%02d/%02d "
                             , ltm->tm_hour    , ltm->tm_min , ltm->tm_sec
                             , ltm->tm_mon + 1 , ltm->tm_mday, ltm->tm_year);

            switch( ich.type) {
                    case ICMP_ECHOREPLY     : strcpy( type, "Echo Reply"             ); break;
                    case ICMP_DEST_UNREACH  : strcpy( type, "Dest. Unreachable"      ); break;
                    case ICMP_SOURCE_QUENCH : strcpy( type, "Source Quench"          ); break;
                    case ICMP_REDIRECT      : strcpy( type, "Redirect ch. route"     ); break;
                    case ICMP_ECHO          : strcpy( type, "Echo Request"           ); break;
                    case 9                  : strcpy( type, "Router Advertisement"   ); break;
                    case 10                 : strcpy( type, "Router Solicitation"    ); break;
                    case ICMP_TIME_EXCEEDED : strcpy( type, "Time Exceeded"          ); break;
                    case ICMP_PARAMETERPROB : strcpy( type, "Parameter problem"      ); break;
                    case ICMP_TIMESTAMP     : strcpy( type, "Timestamp Request"      ); break;
                    case ICMP_TIMESTAMPREPLY: strcpy( type, "Timestamp Reply"        ); break;
                    case ICMP_INFO_REQUEST  : strcpy( type, "Information Request"    ); break;
                    case ICMP_INFO_REPLY    : strcpy( type, "Information Reply"      ); break;
                    case ICMP_ADDRESS       : strcpy( type, "Address Mask Request"   ); break;
                    case ICMP_ADDRESSREPLY  : strcpy( type, "Address Mask Reply"     ); break;
                    case 37                 : strcpy( type, "Domain Name Request"    ); break;
                    case 38                 : strcpy( type, "Domain Name Reply"      ); break;
                    default                 : sprintf( type, "Bad type: %d", ich.type); break;
            } /* end switch( ich.type) */
            
            addrs.s_addr = iph.saddr;
            fprintf( fplog, "%-15.15s (%-17.17s) ", inet_ntoa( addrs), resolve_host_name( iph.saddr));

            fprintf( fplog, "%-20.20s %4d\n", type, ich.code);

            fclose( fplog);
        }

        /* Dump time + raw data */

        if( flag_raw) {
            fpraw = fopen( PRO_ICMP_RAW, "a+");
            fwrite(  &t,          sizeof( t), 1, fpraw);
            fwrite( buf, htons( iph.tot_len), 1, fpraw);
            fclose( fpraw);
        }

 } /* end while( 1) */

 close( sock);

 return 0;
}

void usage( char *progname)
{
 fprintf( stderr, "\nProtolog Release %s - ", PRO_VERSION);
 fprintf( stderr, "plogicmp (ICMP logger)\n\n         by  Diego J. Grigna (diego@grigna.com)\n\n");
 fprintf( stderr, "Usage:\n%s [-lwr]\n", progname);
 fprintf( stderr, "\t-l\t Make a simple human readeable log\n");
 fprintf( stderr, "\t-w\t Log time + raw data\n");
 fprintf( stderr, "\t-r\t Resolve domain names\n\n");
 exit( -1);
}

